# Copyright 2017 - Nokia Corporation
# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
# noinspection PyPackageRequirements
import json
import requests_mock
from unittest import mock
from vitrage.middleware.keycloak import KeycloakAuth
from vitrage.tests.functional.api.v1 import FunctionalTest
from webtest import TestRequest


TOKEN = {
    "iss": "http://127.0.0.1/auth/realms/my_realm",
    "aud": "openstack",
    "realm_access": {
        "roles": ["role1", "role2"]
    }
}

HEADERS = {
    'X-Auth-Token': str(TOKEN),
    'X-Project-Id': 'my_realm'
}


CERT_URL = 'http://127.0.0.1:9080/auth/realms/my_realm/' \
           'protocol/openid-connect/certs'

PUBLIC_KEY = json.loads("""
        {
            "keys": [
                {
                    "kid": "FJ86GcF3jTbNLOco4NvZkUCIUmfYCqoqtOQeMfbhNlE",
                    "kty": "RSA",
                    "alg": "RS256",
                    "use": "sig",
                    "n": "q1awrk7QK24Gmcy9Yb4dMbS-ZnO6",
                    "e": "AQAB"
                }
            ]
        }
        """)

EVENT_DETAILS = {
    'hostname': 'host123',
    'source': 'sample_monitor',
    'cause': 'another alarm',
    'severity': 'critical',
    'status': 'down',
    'monitor_id': 'sample monitor',
    'monitor_event_id': '456',
}

NO_TOKEN_ERROR_MSG = {'error': {
    'code': 401,
    'title': 'Unauthorized',
    'message': 'Auth token must be provided in "X-Auth-Token" header.',
}}


class KeycloakTest(FunctionalTest):

    def __init__(self, *args, **kwds):
        super(KeycloakTest, self).__init__(*args, **kwds)
        self.auth = 'keycloak'

    @staticmethod
    def _build_request():
        req = TestRequest.blank('/')
        req.headers = HEADERS
        return req

    @mock.patch('jwt.decode', return_value=TOKEN)
    @requests_mock.Mocker()
    def test_header_parsing(self, _, req_mock):

        # Imitate success response from KeyCloak.
        req_mock.get(CERT_URL, json=PUBLIC_KEY)

        req = self._build_request()
        auth = KeycloakAuth(mock.Mock(), self.conf)
        auth.process_request(req)

        self.assertEqual('Confirmed', req.headers['X-Identity-Status'])
        self.assertEqual('my_realm', req.headers['X-Project-Id'])
        self.assertEqual('role1,role2', req.headers['X-Roles'])
        self.assertEqual(1, req_mock.call_count)

    def test_in_keycloak_mode_no_token(self):
        resp = self.post_json('/topology/', expect_errors=True)

        self.assertEqual('401 Unauthorized', resp.status)
        self.assert_dict_equal(NO_TOKEN_ERROR_MSG, resp.json)

    @mock.patch('jwt.decode', return_value=TOKEN)
    @requests_mock.Mocker()
    def test_in_keycloak_mode_wrong_token(self, _, req_mock):

        # Imitate failure response from KeyCloak.
        req_mock.get(
            requests_mock.ANY,
            status_code=401,
            reason='Access token is invalid'
        )

        resp = self.post_json('/topology/',
                              params=None,
                              headers=HEADERS,
                              expect_errors=True)

        self.assertEqual('401 Unauthorized', resp.status)

    @mock.patch('jwt.decode', return_value=TOKEN)
    @requests_mock.Mocker()
    def test_in_keycloak_mode_auth_success(self, _, req_mock):

        # Imitate success response from KeyCloak.
        req_mock.get(CERT_URL, json=PUBLIC_KEY)

        with mock.patch('pecan.request') as request:
            resp = self.post_json('/event/',
                                  params={
                                      'time': datetime.now().isoformat(),
                                      'type': 'compute.host.down',
                                      'details': EVENT_DETAILS
                                  },
                                  headers=HEADERS)

            self.assertEqual(1, request.client.call.call_count)
            self.assertEqual('200 OK', resp.status)



# Copyright 2018 Samsung Electronics
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.alarm_transformer_base import AlarmTransformerBase
from vitrage.datasources.monasca import MONASCA_DATASOURCE
from vitrage.datasources.monasca.properties import (
    MonascaAlarmStatuses as MAlarmStatuses)
from vitrage.datasources.monasca.properties import MonascaProperties as MProps
from vitrage.datasources import transformer_base as tbase
from vitrage.datasources.transformer_base import extract_field_value
import vitrage.graph.utils as graph_utils


class MonascaTransformer(AlarmTransformerBase):

    def _create_snapshot_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_update_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_vertex(self, entity_event):
        update_timestamp = entity_event[MProps.UPDATE_TIMESTAMP]
        sample_timestamp = entity_event[DSProps.SAMPLE_DATE]

        name = extract_field_value(
            entity_event, 'alarm_definition', 'name')

        metadata = {
            VProps.NAME: name,
            VProps.VITRAGE_RESOURCE_ID: entity_event[MProps.RESOURCE_ID],
            VProps.VITRAGE_RESOURCE_TYPE: entity_event[MProps.RESOURCE_TYPE],
            VProps.SEVERITY: entity_event[MProps.STATUS]
        }

        return graph_utils.create_vertex(
            self._create_entity_key(entity_event),
            vitrage_category=EntityCategory.ALARM,
            vitrage_type=MONASCA_DATASOURCE,
            vitrage_sample_timestamp=sample_timestamp,
            entity_id=entity_event[MProps.ID],
            entity_state=self._get_alarm_state(entity_event),
            update_timestamp=update_timestamp,
            metadata=metadata)

    def _create_snapshot_neighbors(self, entity_event):
        return self._create_monasca_neighbors(entity_event)

    def _create_update_neighbors(self, entity_event):
        return self._create_monasca_neighbors(entity_event)

    def _create_monasca_neighbors(self, entity_event):
        return [self._create_neighbor(
            entity_event,
            entity_event[MProps.RESOURCE_ID],
            entity_event[MProps.RESOURCE_TYPE],
            EdgeLabel.ON,
            neighbor_category=EntityCategory.RESOURCE)]

    def _ok_status(self, entity_event):
        return entity_event[MProps.STATUS] == MAlarmStatuses.OK

    def _create_entity_key(self, entity_event):
        entity_id = entity_event[MProps.ID]
        key_fields = self._key_values(MONASCA_DATASOURCE, entity_id)
        return tbase.build_key(key_fields)

    def get_vitrage_type(self):
        return MONASCA_DATASOURCE



#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_policy import policy

from vitrage.common.policies import base

SERVICE = 'get service list'

rules = [
    policy.DocumentedRuleDefault(
        name=SERVICE,
        check_str=base.UNPROTECTED,
        description='Get the vitrage services for the OpenStack cluster',
        operations=[
            {
                'path': '/service',
                'method': 'GET'
            }
        ]
    )
]


def list_rules():
    return rules



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import GraphAction
from vitrage.datasources.cinder.volume import CINDER_VOLUME_DATASOURCE
from vitrage.datasources.cinder.volume.driver import CinderVolumeDriver
from vitrage.datasources.driver_base import DriverBase
from vitrage.datasources.heat.stack import HEAT_STACK_DATASOURCE
from vitrage.datasources.neutron.network.driver import NetworkDriver
from vitrage.datasources.neutron.network import NEUTRON_NETWORK_DATASOURCE
from vitrage.datasources.neutron.port.driver import PortDriver
from vitrage.datasources.neutron.port import NEUTRON_PORT_DATASOURCE
from vitrage.datasources.nova.instance.driver import InstanceDriver
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage import os_clients

CONF = cfg.CONF


class HeatStackDriver(DriverBase):

    RESOURCE_TYPE = {
        'OS::Nova::Server': NOVA_INSTANCE_DATASOURCE,
        'OS::Cinder::Volume': CINDER_VOLUME_DATASOURCE,
        'OS::Neutron::Net': NEUTRON_NETWORK_DATASOURCE,
        'OS::Neutron::Port': NEUTRON_PORT_DATASOURCE
    }

    RESOURCE_DRIVERS = {
        'OS::Nova::Server': InstanceDriver,
        'OS::Cinder::Volume': CinderVolumeDriver,
        'OS::Neutron::Net': NetworkDriver,
        'OS::Neutron::Port': PortDriver
    }

    def __init__(self):
        super(HeatStackDriver, self).__init__()
        self._client = None
        self._filter_resource_types()

    @property
    def client(self):
        if not self._client:
            self._client = os_clients.heat_client()
        return self._client

    @staticmethod
    def get_topic():
        return CONF[HEAT_STACK_DATASOURCE].notification_topic

    @staticmethod
    def get_event_types():
        return ['orchestration.stack.create.end',
                'orchestration.stack.delete.end',
                'orchestration.stack.update.error',
                'orchestration.stack.update.end',
                'orchestration.stack.suspend.error',
                'orchestration.stack.suspend.end',
                'orchestration.stack.resume.error',
                'orchestration.stack.resume.end']

    def enrich_event(self, event, event_type):
        # TODO(Nofar): add call to get resources of the stack if not deleted
        # change transformer that if delete we remove the stack from the graph
        # and hence all the edges to it

        stack_id = event['stack_identity']
        if self._is_nested_stack(stack_id):
            return

        event[DSProps.EVENT_TYPE] = event_type
        event = self._retrieve_stack_resources(event, stack_id)

        return HeatStackDriver.make_pickleable(
            [event],
            HEAT_STACK_DATASOURCE,
            DatasourceAction.UPDATE,
            *self.properties_to_filter_out())[0]

    def _is_nested_stack(self, _id):
        return self.client.stacks.get(_id).to_dict()['parent']

    def _filter_resource_types(self):
        types = CONF.datasources.types

        self.RESOURCE_TYPE = {key: value for key, value in
                              self.RESOURCE_TYPE.items() if value in types}

    @staticmethod
    def _make_stacks_list(stacks):
        return [stack.to_dict() for stack in stacks]

    def _append_stacks_resources(self, stacks):
        return [self._retrieve_stack_resources(stack, stack['id'])
                for stack in stacks]

    @staticmethod
    def properties_to_filter_out():
        return ['manager', '_info']

    def _retrieve_stack_resources(self, stack, stack_id):
        # guess 10 is enough
        resources = self.client.resources.list(stack_id, nested_depth=10)
        stack['resources'] = [resource.to_dict() for resource in resources
                              if resource.to_dict()['resource_type'] in
                              self.RESOURCE_TYPE]
        self._filter_stack_resources(stack)
        return stack

    @staticmethod
    def _filter_stack_resources(stack):
        for resource in stack['resources']:
            props = HeatStackDriver.RESOURCE_DRIVERS[
                resource['resource_type']].properties_to_filter_out()
            for prop in props:
                if prop in resource:
                    del resource[prop]

    def get_all(self, datasource_action):
        stacks = self.client.stacks.list(global_tenant=True)
        stacks_list = self._make_stacks_list(stacks)
        stacks_with_resources = self._append_stacks_resources(stacks_list)
        for s in stacks_with_resources:
            if s['stack_status'].lower() in ['delete_complete', 'deleted',
                                             'delete_in_progress']:
                s[DSProps.EVENT_TYPE] = GraphAction.DELETE_ENTITY
        return self.make_pickleable(stacks_with_resources,
                                    HEAT_STACK_DATASOURCE,
                                    datasource_action,
                                    *self.properties_to_filter_out())

    @staticmethod
    def should_delete_outdated_entities():
        return True



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,  software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND,  either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import GraphAction
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources.nova.nova_driver_base import NovaDriverBase


CONF = cfg.CONF

# versioned notifications
VERSIONED_NOTIFICATIONS = {
    'instance.create.end',
    'instance.create.error',
    'instance.delete.end',
    'instance.delete.start',
    'instance.evacuate',
    'instance.interface_attach.end',
    'instance.interface_attach.error',
    'instance.interface_detach.end',
    'instance.live_migration_abort.end',
    'instance.live_migration_force_complete.end',
    'instance.live_migration_post.end',
    'instance.live_migration_post_dest.end',
    'instance.live_migration_rollback.end',
    'instance.live_migration_rollback_dest.end',
    'instance.lock',
    'instance.pause.end',
    'instance.power_off.end',
    'instance.power_on.end',
    'instance.reboot.end',
    'instance.reboot.error',
    'instance.rebuild.end',
    'instance.rebuild.error',
    'instance.rescue.end',
    'instance.resize.end',
    'instance.resize.error',
    'instance.resize_confirm.end',
    'instance.resize_finish.end',
    'instance.resize_prep.end',
    'instance.resize_revert.end',
    'instance.restore.end',
    'instance.resume.end',
    'instance.shelve.end',
    'instance.shelve_offload.end',
    'instance.shutdown.end',
    'instance.soft_delete.end',
    'instance.snapshot.end',
    'instance.suspend.end',
    'instance.unlock',
    'instance.unpause.end',
    'instance.unrescue.end',
    'instance.unshelve.end',
    'instance.update',
    'instance.volume_attach.end',
    'instance.volume_attach.error',
    'instance.volume_detach.end',
    'instance.volume_swap.end',
    'instance.volume_swap.error',
}

# legacy (unversioned) notifications
LEGACY_NOTIFICATIONS = {
    'compute.instance.create.error',
    'compute.instance.create.end',
    'compute.instance.delete.start',
    'compute.instance.delete.end',
    'compute.instance.finish_resize.end',
    'compute.instance.live_migration.post.dest.end',
    'compute.instance.live_migration._post.end',
    'compute.instance.power_off.end',
    'compute.instance.power_on.end',
    'compute.instance.reboot.end',
    'compute.instance.rebuild.end',
    'compute.instance.resize.end',
    'compute.instance.resize.revert.end',
    'compute.instance.resume.end',
    'compute.instance.shutdown.end',
    'compute.instance.suspend.end',
    'compute.instance.volume.attach',
    'compute.instance.volume.detach',
    'compute.instance.pause.end',
    'compute.instance.unpause.end'
}


class InstanceDriver(NovaDriverBase):

    @staticmethod
    def extract_events(instances):
        events = [vars(instance) for instance in instances]
        for e in events:
            if e['status'].lower() == 'deleted':
                e[DSProps.EVENT_TYPE] = GraphAction.DELETE_ENTITY
        return events

    def get_all(self, datasource_action):
        return self.make_pickleable(
            self.extract_events(self.client.servers.list(
                search_opts={'all_tenants': 1})),
            NOVA_INSTANCE_DATASOURCE,
            datasource_action,
            *self.properties_to_filter_out())

    def enrich_event(self, event, event_type):
        use_versioned = CONF.use_nova_versioned_notifications

        # Send to the processor only events of the matching types. Nova may
        # send both versioned and legacy notifications, and we don't want to
        # handle a similar event twice.
        if (use_versioned and event_type in VERSIONED_NOTIFICATIONS) or \
                ((not use_versioned) and event_type in LEGACY_NOTIFICATIONS):
            event[DSProps.EVENT_TYPE] = event_type
            return InstanceDriver.make_pickleable([event],
                                                  NOVA_INSTANCE_DATASOURCE,
                                                  DatasourceAction.UPDATE)[0]

        return []

    @staticmethod
    def properties_to_filter_out():
        return ['manager', 'OS-EXT-SRV-ATTR:user_data', '_info']

    @staticmethod
    def get_event_types():
        return list(VERSIONED_NOTIFICATIONS | LEGACY_NOTIFICATIONS)

    @staticmethod
    def should_delete_outdated_entities():
        return True



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from vitrage.evaluator.template_validation.content.v2.\
    execute_mistral_validator import ExecuteMistralValidator as \
    V2ExecuteMistralValidator
from vitrage.tests.unit.evaluator.template_validation.content.\
    base_test_execute_mistral_validator import BaseExecuteMistralValidatorTest
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    DEFINITIONS_INDEX_MOCK


class ExecuteMistralValidatorTest(BaseExecuteMistralValidatorTest):

    @classmethod
    def setUpClass(cls):
        super(ExecuteMistralValidatorTest, cls).setUpClass()
        cls.validator = V2ExecuteMistralValidator()

    def test_v2_validate_execute_mistral_action(self):
        self._validate_execute_mistral_action(self.validator)

    def test_v2_validate_old_execute_mistral_action(self):
        """Test version2 validator on version1 template.

        An execute_mistral action from version 1 should fail in the validation
        of version 2.
        """

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        v1_action = \
            self._create_v1_execute_mistral_action('wf_1', 'host_2', 'down')

        # Test action
        result = self.validator.validate(v1_action, idx)

        # Test assertions
        self._assert_fault_result(result, 136)

    def test_validate_execute_mistral_action_without_workflow(self):
        self._validate_execute_mistral_action_without_workflow(self.validator)

    def test_validate_execute_mistral_action_with_empty_workflow(self):
        self._validate_execute_mistral_action_with_empty_workflow(
            self.validator)

    def test_validate_execute_mistral_action_with_none_workflow(self):
        self._validate_execute_mistral_action_with_none_workflow(
            self.validator)

    def test_validate_execute_mistral_action_without_additional_props(self):
        self._validate_execute_mistral_action_without_additional_props(
            self.validator)

    def test_v2_validate_execute_mistral_action_with_func(self):
        self._validate_action(
            self._create_v2_execute_mistral_action(
                'wf_1', 'host_2', 'down', func1='get_attr(alarm,name)'),
            self.validator.validate
        )

    def test_v2_validate_execute_mistral_action_with_func_2(self):
        self._validate_action(
            self._create_v2_execute_mistral_action(
                'wf_1', 'host_2', 'down', func1='get_attr(alarm, name)'),
            self.validator.validate
        )

    def test_v2_validate_execute_mistral_action_with_func_3(self):
        self._validate_action(
            self._create_v2_execute_mistral_action(
                'wf_1', 'host_2', 'down', func1='get_attr ( alarm , name ) '),
            self.validator.validate
        )

    def test_v2_validate_execute_mistral_action_with_func_typo_1(self):
        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = \
            self._create_v2_execute_mistral_action(
                'wf_1', 'host_2', 'down', func1='get_attr(alarm, name')

        # Test action
        result = self.validator.validate(action, idx)

        # Test assertions
        self._assert_warning_result(result, 138)

    def test_v2_validate_execute_mistral_action_with_func_typo_2(self):
        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = \
            self._create_v2_execute_mistral_action(
                'wf_1', 'host_2', 'down', func1='get_attr, name)')

        # Test action
        result = self.validator.validate(action, idx)

        # Test assertions
        self._assert_warning_result(result, 138)

    def _create_execute_mistral_action(self, workflow, host, host_state):
        return self.\
            _create_v2_execute_mistral_action(workflow, host, host_state)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from oslo_log import log
from osprofiler import profiler

from vitrage.common.constants import TemplateStatus as TStatus
from vitrage.evaluator.template_db import template_repository as template_repo
from vitrage.evaluator.template_schema_factory import TemplateSchemaFactory


LOG = log.getLogger(__name__)


@profiler.trace_cls("template apis",
                    info={}, hide_args=False, trace_private=False)
class TemplateApis(object):

    FAILED_MSG = 'validation failed'
    OK_MSG = 'validation OK'

    def __init__(self, notifier=None, db=None):
        self.notifier = notifier
        self.db = db

    def validate_template(self, ctx, templates, template_type, params=None):
        LOG.debug("TemplateApis validate_template type: %s content: %s",
                  template_type, templates)

        files_content = [t[1] for t in templates]
        paths = [t[0] for t in templates]
        results = template_repo.validate_templates(self.db, files_content,
                                                   template_type, params)
        results = [_to_result(r, p) for r, p in zip(results, paths)]
        return json.dumps({'results': results})

    def template_versions(self, ctx):
        versions = sorted(TemplateSchemaFactory.supported_versions())
        # TODO(eyalb) at the moment all are supported
        current = max(versions)
        return [
            {
                'version': 'v%s' % version,
                'status': 'SUPPORTED' if version < current else 'CURRENT'
            }
            for version in versions
        ]

    def add_template(self, ctx, templates, template_type, params=None):
        """Signal the evaluator

         A new template has been added to the database with a status of
         LOADING that needs to be handled.
        """
        LOG.debug("TemplateApis add_template type: %s content: %s params: %s",
                  template_type, templates, params)

        files_content = [t[1] for t in templates]
        db_rows = template_repo.add_templates_to_db(self.db, files_content,
                                                    template_type, params)
        if self._is_evaluator_reload_required(db_rows):
            LOG.info("Adding templates..")
            self.notifier.notify("add template", {'template_action': 'add'})

        return [_db_template_to_dict(r) for r in db_rows]

    def _is_evaluator_reload_required(self, db_rows):
        """Is  evaluator reload required

        If all the templates have error status, no need to reload evaluators
        """
        return any([True for t in db_rows if t.status != TStatus.ERROR])

    def delete_template(self, ctx, uuids):
        """Signal the evaluator

         A template status has been changed to DELETING.
        """
        db = self.db

        if type(uuids) != list:
            uuids = [uuids]
        LOG.info("Deleting templates %s ", uuids)
        templates = [t for _id in uuids for t in db.templates.query(uuid=_id)
                     if t.status != TStatus.DELETED]
        if not templates:
            return
        for t in templates:
            if t.status == TStatus.ERROR:
                db.templates.update(t.uuid, "status", TStatus.DELETED)
            else:
                db.templates.update(t.uuid, "status", TStatus.DELETING)
        if self._is_evaluator_reload_required(templates):
            self.notifier.notify("delete template",
                                 {'template_action': 'delete'})


def _to_result(result, template_path):
    if result.is_valid_config:
        return {
            'file path': template_path,
            'status': TemplateApis.OK_MSG,
            'description': 'Template validation',
            'message': str(result.comment),
            'status code': result.status_code
        }
    else:
        return {
            'file path': template_path,
            'status': TemplateApis.FAILED_MSG,
            'description': result.description,
            'message': str(result.comment),
            'status code': result.status_code
        }


def _db_template_to_dict(template):
    return {
        "uuid": template.uuid,
        "name": template.name,
        "status": template.status,
        "date": str(template.created_at),
        "status details": template.status_details,
        "type": template.template_type,
    }



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_vitrage graph
----------------------------------

Tests for `vitrage` graph driver
"""

import copy

from vitrage.common.constants import EntityCategory
from vitrage.common.constants import NotifierEventTypes as NType
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.entity_graph.processor.notifier import GraphNotifier as GN
from vitrage.evaluator.actions import evaluator_event_transformer as evaluator
from vitrage.graph import Vertex
from vitrage.tests import base


resource = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
    VProps.VITRAGE_TYPE: 'some_resource_type',
    VProps.VITRAGE_IS_DELETED: False,
    VProps.VITRAGE_IS_PLACEHOLDER: False,
    })

deduced_alarm = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.ALARM,
    VProps.VITRAGE_TYPE: evaluator.VITRAGE_DATASOURCE,
    VProps.VITRAGE_IS_DELETED: False,
    VProps.VITRAGE_IS_PLACEHOLDER: False,
    })


non_deduced_alarm = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.ALARM,
    VProps.VITRAGE_TYPE: 'TEST_ALARM',
    VProps.VITRAGE_IS_DELETED: False,
    VProps.VITRAGE_IS_PLACEHOLDER: True,
    })

deleted_alarm = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.ALARM,
    VProps.VITRAGE_TYPE: evaluator.VITRAGE_DATASOURCE,
    VProps.VITRAGE_IS_DELETED: True,
    VProps.VITRAGE_IS_PLACEHOLDER: False,
    })

placeholder_alarm = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.ALARM,
    VProps.VITRAGE_TYPE: evaluator.VITRAGE_DATASOURCE,
    VProps.VITRAGE_IS_DELETED: False,
    VProps.VITRAGE_IS_PLACEHOLDER: True,
    })


host = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
    VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE,
    VProps.VITRAGE_IS_DELETED: False,
    VProps.VITRAGE_IS_PLACEHOLDER: False,
    })

forced_down_host = Vertex('123', {
    VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
    VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE,
    VProps.VITRAGE_IS_DELETED: False,
    VProps.VITRAGE_IS_PLACEHOLDER: False,
    VProps.IS_MARKED_DOWN: True,
    })


class GraphTest(base.BaseTest):
    def get_first(self, lst):
        self.assertIsNotNone(lst)
        return lst[0] if len(lst) > 0 else None

    def test_notification_type_new_alarm(self):
        ret = GN._get_notification_type(None, deduced_alarm, True)
        self.assertEqual(NType.ACTIVATE_DEDUCED_ALARM_EVENT,
                         self.get_first(ret),
                         'new alarm should notify activate')

        ret = GN._get_notification_type(None, non_deduced_alarm, True)
        self.assertIsNone(self.get_first(ret),
                          'alarm that is not a deduced alarm')

    def test_notification_type_deleted_alarm(self):
        ret = GN._get_notification_type(deduced_alarm, deleted_alarm, True)
        self.assertEqual(NType.DEACTIVATE_DEDUCED_ALARM_EVENT,
                         self.get_first(ret),
                         'deleted alarm should notify deactivate')

    def test_notification_type_resource_vertex(self):
        ret = GN._get_notification_type(None, resource, True)
        self.assertIsNone(self.get_first(ret),
                          'any non alarm vertex should be ignored')

    def test_notification_type_updated_alarm(self):
        ret = GN._get_notification_type(deduced_alarm, deduced_alarm, True)
        self.assertIsNone(self.get_first(ret),
                          'A not new alarm vertex should be ignored')

        ret = GN._get_notification_type(deleted_alarm, deduced_alarm, True)
        self.assertEqual(NType.ACTIVATE_DEDUCED_ALARM_EVENT,
                         self.get_first(ret),
                         'old alarm become not deleted should notify activate')

        ret = GN._get_notification_type(placeholder_alarm, deduced_alarm, True)
        self.assertEqual(NType.ACTIVATE_DEDUCED_ALARM_EVENT,
                         self.get_first(ret),
                         'placeholder become active should notify activate')

    def test_notification_type_placeholder_alarm(self):
        ret = GN._get_notification_type(None, placeholder_alarm, True)
        self.assertIsNone(self.get_first(ret),
                          'A not new alarm vertex should be ignored')

    def test_notification_type_new_host(self):
        ret = GN._get_notification_type(None, forced_down_host, True)
        self.assertEqual(NType.ACTIVATE_MARK_DOWN_EVENT,
                         self.get_first(ret),
                         'new host with forced_down should notify activate')

        ret = GN._get_notification_type(None, host, True)
        self.assertIsNone(self.get_first(ret), 'host without forced_down')

    def test_notification_type_deleted_host(self):
        deleted_host = copy.deepcopy(forced_down_host)
        deleted_host[VProps.VITRAGE_IS_DELETED] = True
        ret = GN._get_notification_type(forced_down_host, deleted_host, True)
        self.assertEqual(
            NType.DEACTIVATE_MARK_DOWN_EVENT,
            self.get_first(ret),
            'deleted host with forced_down should notify deactivate')

        deleted_host = copy.deepcopy(host)
        deleted_host[VProps.VITRAGE_IS_DELETED] = True
        ret = GN._get_notification_type(forced_down_host, deleted_host, True)
        self.assertEqual(
            NType.DEACTIVATE_MARK_DOWN_EVENT,
            self.get_first(ret),
            'deleted host with forced_down should notify deactivate')

        deleted_host = copy.deepcopy(host)
        deleted_host[VProps.VITRAGE_IS_DELETED] = True
        ret = GN._get_notification_type(host, deleted_host, True)
        self.assertIsNone(
            self.get_first(ret),
            'deleted host without forced_down should not notify')

    def test_notification_type_updated_host(self):
        ret = GN._get_notification_type(
            forced_down_host, forced_down_host, True)
        self.assertIsNone(self.get_first(ret),
                          'A not new host should be ignored')

        deleted_host = copy.deepcopy(forced_down_host)
        deleted_host[VProps.VITRAGE_IS_DELETED] = True
        ret = GN._get_notification_type(deleted_host, forced_down_host, True)
        self.assertEqual(NType.ACTIVATE_MARK_DOWN_EVENT,
                         self.get_first(ret),
                         'old host become not deleted should notify activate')

        deleted_host = copy.deepcopy(forced_down_host)
        deleted_host[VProps.VITRAGE_IS_DELETED] = True
        ret = GN._get_notification_type(deleted_host, host, True)
        self.assertIsNone(self.get_first(ret),
                          'old host become not deleted should not notify')

        placeholder_host = copy.deepcopy(forced_down_host)
        placeholder_host[VProps.VITRAGE_IS_PLACEHOLDER] = True
        ret = GN._get_notification_type(
            placeholder_host, forced_down_host, True)
        self.assertEqual(NType.ACTIVATE_MARK_DOWN_EVENT,
                         self.get_first(ret),
                         'placeholder become active should notify activate')

    def test_notification_type_placeholder_host(self):
        placeholder_host = copy.deepcopy(forced_down_host)
        placeholder_host[VProps.VITRAGE_IS_PLACEHOLDER] = True
        ret = GN._get_notification_type(None, placeholder_host, True)
        self.assertIsNone(self.get_first(ret),
                          'A not new host vertex should be ignored')



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.tests.mocks import mock_driver
from vitrage.tests.unit.entity_graph.base import TestEntityGraphUnitBase


class TestFunctionalBase(TestEntityGraphUnitBase):

    def _create_processor_with_graph(self, processor=None):
        events = self._create_mock_events()

        if not processor:
            processor = self.create_processor_and_graph()

        for event in events:
            processor.process_event(event)

        return processor

    def _create_mock_events(self):
        gen_list = mock_driver.simple_zone_generators(
            self.NUM_ZONES,
            self.NUM_HOSTS,
            snapshot_events=self.NUM_ZONES,
            snap_vals={DSProps.DATASOURCE_ACTION:
                       DatasourceAction.INIT_SNAPSHOT})
        gen_list += mock_driver.simple_host_generators(
            self.NUM_ZONES,
            self.NUM_HOSTS,
            self.NUM_HOSTS,
            snap_vals={DSProps.DATASOURCE_ACTION:
                       DatasourceAction.INIT_SNAPSHOT})
        gen_list += mock_driver.simple_instance_generators(
            host_num=self.NUM_HOSTS,
            vm_num=self.NUM_INSTANCES,
            snapshot_events=self.NUM_INSTANCES,
            snap_vals={DSProps.DATASOURCE_ACTION:
                       DatasourceAction.INIT_SNAPSHOT})
        return mock_driver.generate_sequential_events_list(gen_list)

    @staticmethod
    def _consume_queue(event_queue, processor):
        while not event_queue.empty():
            data = event_queue.get()
            if isinstance(data, list):
                for event in data:
                    processor.process_event(event)
            else:
                processor.process_event(data)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.entity_graph.mappings.operational_resource_state import \
    OperationalResourceState
from vitrage.evaluator.actions.base import ActionType
from vitrage.evaluator.template_fields import TemplateFields
from vitrage.evaluator.template_validation.content.v1.set_state_validator \
    import SetStateValidator
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    ActionValidatorTest
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    DEFINITIONS_INDEX_MOCK


class SetStateValidatorTest(ActionValidatorTest):

    def test_validate_set_state_action(self):

        self._validate_action(self._create_set_state_action('123'),
                              SetStateValidator.validate)

    def test_validate_set_state_action_without_action_target(self):

        self._validate_action_without_action_target(
            self._create_set_state_action('123'),
            SetStateValidator.validate
        )

    def test_validate_set_state_action_with_invalid_target_id(self):

        self._validate_action_with_invalid_target_id(
            self._create_set_state_action('unknown'),
            SetStateValidator.validate
        )

    def test_validate_set_state_action_without_target_id(self):

        self._validate_action_without_target_id(
            self._create_set_state_action('123'),
            SetStateValidator.validate,
            129
        )

    def test_validate_set_state_action_without_state_property(self):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = self._create_set_state_action('123')
        action[TemplateFields.PROPERTIES].pop(TemplateFields.STATE, None)

        # Test action
        result = SetStateValidator.validate(action, idx)

        # Test assertions
        self._assert_fault_result(result, 128)

    @staticmethod
    def _create_set_state_action(target):

        action_target = {
            TemplateFields.TARGET: target
        }
        properties = {
            TemplateFields.STATE: OperationalResourceState.SUBOPTIMAL
        }
        action = {
            TemplateFields.ACTION_TYPE: ActionType.SET_STATE,
            TemplateFields.ACTION_TARGET: action_target,
            TemplateFields.PROPERTIES: properties
        }
        return action



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.alarm_transformer_base import AlarmTransformerBase
from vitrage.datasources.nagios import NAGIOS_DATASOURCE
from vitrage.datasources.nagios.properties import NagiosProperties
from vitrage.datasources.nagios.properties import NagiosTestStatus
from vitrage.datasources import transformer_base as tbase
import vitrage.graph.utils as graph_utils
from vitrage.utils import datetime as datetime_utils


class NagiosTransformer(AlarmTransformerBase):

    def _create_snapshot_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_update_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_vertex(self, entity_event):

        update_timestamp = datetime_utils.change_to_utc_time_and_format(
            entity_event[NagiosProperties.LAST_CHECK],
            tbase.TIMESTAMP_FORMAT)
        vitrage_sample_timestamp = entity_event[DSProps.SAMPLE_DATE]
        update_timestamp = \
            self._format_update_timestamp(update_timestamp,
                                          vitrage_sample_timestamp)

        metadata = {
            VProps.NAME: entity_event[NagiosProperties.SERVICE],
            VProps.RESOURCE_ID: entity_event[NagiosProperties.RESOURCE_NAME],
            VProps.SEVERITY: entity_event[NagiosProperties.STATUS],
            VProps.INFO: entity_event[NagiosProperties.STATUS_INFO]
        }

        return graph_utils.create_vertex(
            self._create_entity_key(entity_event),
            vitrage_category=EntityCategory.ALARM,
            vitrage_type=entity_event[DSProps.ENTITY_TYPE],
            vitrage_sample_timestamp=vitrage_sample_timestamp,
            entity_state=self._get_alarm_state(entity_event),
            update_timestamp=update_timestamp,
            metadata=metadata)

    def _create_snapshot_neighbors(self, entity_event):
        return self._create_nagios_neighbors(entity_event)

    def _create_update_neighbors(self, entity_event):
        return self._create_nagios_neighbors(entity_event)

    def _create_nagios_neighbors(self, entity_event):
        resource_type = entity_event[NagiosProperties.RESOURCE_TYPE]
        if resource_type:
            return [self._create_neighbor(
                entity_event,
                entity_event[NagiosProperties.RESOURCE_NAME],
                resource_type,
                EdgeLabel.ON,
                neighbor_category=EntityCategory.RESOURCE)]

        return []

    def _ok_status(self, entity_event):
        return entity_event[NagiosProperties.STATUS] == NagiosTestStatus.OK

    def _create_entity_key(self, entity_event):
        entity_type = entity_event[DSProps.ENTITY_TYPE]
        alarm_name = entity_event[NagiosProperties.SERVICE]
        resource_name = entity_event[NagiosProperties.RESOURCE_NAME]
        return tbase.build_key((EntityCategory.ALARM,
                                entity_type,
                                resource_name,
                                alarm_name))

    def get_vitrage_type(self):
        return NAGIOS_DATASOURCE



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_config import cfg
from oslo_log import log as logging
from testtools import matchers

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import GraphAction
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.kubernetes.properties import KUBERNETES_DATASOURCE
from vitrage.datasources.kubernetes.properties import KubernetesProperties \
    as kubProp
from vitrage.datasources.kubernetes.transformer import KubernetesTransformer
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources.nova.instance.transformer import InstanceTransformer
from vitrage.datasources import transformer_base as tbase
from vitrage.datasources.transformer_base import TransformerBase
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver as mock_sync
from vitrage.tests.mocks import utils

LOG = logging.getLogger(__name__)

cluster_name = 'kubernetes'


class KubernetesTransformerTest(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PULL),
        cfg.StrOpt(DSOpts.CONFIG_FILE,
                   default=utils.get_resources_dir() +
                   '/kubernetes/kubernetes_config.yaml'),
    ]

    # noinspection PyAttributeOutsideInit,PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(KubernetesTransformerTest, cls).setUpClass()
        cls.transformers = {}
        cls.transformers[KUBERNETES_DATASOURCE] = KubernetesTransformer(
            cls.transformers)
        cls.transformers[NOVA_INSTANCE_DATASOURCE] = \
            InstanceTransformer(cls.transformers)

    def setUp(self):
        super(KubernetesTransformerTest, self).setUp()
        self.conf_reregister_opts(self.OPTS, group=KUBERNETES_DATASOURCE)

    def test_snapshot_event_transform(self):
        LOG.debug('Test tactual transform action for '
                  'snapshot and snapshot init events')

        k8s_spec_list = \
            mock_sync.simple_k8s_nodes_generators(nodes_num=2,
                                                  snapshot_events=1)

        nodes_events = mock_sync.generate_random_events_list(k8s_spec_list)

        for event in nodes_events:

            k8s_wrapper = self.transformers[KUBERNETES_DATASOURCE].transform(
                event)

            # Test assertions
            self.assertEqual(cluster_name, k8s_wrapper.vertex[VProps.NAME])
            n_length = str(len(k8s_wrapper.neighbors))
            self.assertThat(n_length, matchers.HasLength(1),
                            'Cluster vertex has one neighbor')
            self._validate_cluster_neighbors(k8s_wrapper.neighbors, event)
            datasource_action = event[DSProps.DATASOURCE_ACTION]
            if datasource_action == DatasourceAction.INIT_SNAPSHOT:
                self.assertEqual(GraphAction.CREATE_ENTITY, k8s_wrapper.action)
            elif datasource_action == DatasourceAction.SNAPSHOT:
                self.assertEqual(GraphAction.UPDATE_ENTITY, k8s_wrapper.action)

    def test_build_cluster_key(self):
        LOG.debug('Test build cluster key')

        # Test setup
        expected_key = 'RESOURCE:kubernetes:kubernetes'

        instance_transformer = self.transformers[NOVA_INSTANCE_DATASOURCE]
        # Test action
        key_fields = instance_transformer._key_values(
            KUBERNETES_DATASOURCE,
            cluster_name)

        # Test assertions
        observed_key = tbase.build_key(key_fields)
        self.assertEqual(expected_key, observed_key)

    def _validate_cluster_neighbors(self, neighbor, event):

        # Create expected neighbor
        time = event[DSProps.SAMPLE_DATE]
        external_id = event['resources'][0][kubProp.EXTERNALID]
        properties = {
            VProps.ID: external_id,
            VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE,
            VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
            VProps.VITRAGE_SAMPLE_TIMESTAMP: time
        }
        nova_instance_tran = self.transformers[NOVA_INSTANCE_DATASOURCE]
        expected_neighbor = \
            nova_instance_tran.create_neighbor_placeholder_vertex(**properties)
        self.assertEqual(expected_neighbor, neighbor[0].vertex)

        # Validate neighbor edge
        edge = neighbor[0].edge
        entity_key = \
            self.transformers[KUBERNETES_DATASOURCE]._create_entity_key(event)
        entity_uuid = \
            TransformerBase.uuid_from_deprecated_vitrage_id(entity_key)
        self.assertEqual(edge.source_id, entity_uuid)
        self.assertEqual(edge.target_id, neighbor[0].vertex.vertex_id)

    def test_create_entity_key(self):
        LOG.debug('Test get key from kubernetes transformer')

        # Test setup
        spec_list = mock_sync.simple_k8s_nodes_generators(nodes_num=1,
                                                          snapshot_events=1)

        nodes_events = mock_sync.generate_random_events_list(spec_list)

        kubernetes_transformer = self.transformers[KUBERNETES_DATASOURCE]
        for event in nodes_events:
            # Test action
            observed_key = kubernetes_transformer._create_entity_key(event)

            # Test assertions
            observed_key_fields = observed_key.split(
                TransformerBase.KEY_SEPARATOR)

            self.assertEqual(EntityCategory.RESOURCE, observed_key_fields[0])
            self.assertEqual(
                KUBERNETES_DATASOURCE,
                observed_key_fields[1]
            )

            key_values = kubernetes_transformer._key_values(
                KUBERNETES_DATASOURCE,
                cluster_name)
            expected_key = tbase.build_key(key_values)

            self.assertEqual(expected_key, observed_key)



# Copyright 2018 Samsung Electronics
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import UpdateMethod

MONASCA_DATASOURCE = 'monasca'

OPTS = [
    cfg.StrOpt(DSOpts.TRANSFORMER,
               default='vitrage.datasources.monasca.transformer.'
                       'MonascaTransformer',
               help='Monasca transformer class path.',
               required=True),
    cfg.StrOpt(DSOpts.DRIVER,
               default='vitrage.datasources.monasca.driver.'
                       'MonascaDriver',
               help='Monasca driver class path.',
               required=True),
    cfg.StrOpt(DSOpts.UPDATE_METHOD,
               default=UpdateMethod.PULL,
               help='None: updates only via Vitrage periodic snapshots.'
                    'Pull: updates periodically.'
                    'Push: updates by getting notifications from the'
                    ' datasource itself.',
               required=True),
    cfg.IntOpt(DSOpts.CHANGES_INTERVAL,
               default=30,
               min=10,
               help='Interval in seconds between checking changes in Monasca'
                    'datasource.'),
    ]



# Copyright 2016 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import pecan

from oslo_log import log
from oslo_utils.strutils import bool_from_string
from osprofiler import profiler
from pecan.core import abort
from vitrage.api.controllers.rest import RootRestController
from vitrage.api.policy import enforce


LOG = log.getLogger(__name__)


# noinspection PyBroadException
@profiler.trace_cls("rca controller",
                    info={}, hide_args=False, trace_private=False)
class RCAController(RootRestController):
    @pecan.expose('json')
    def index(self, alarm_id, all_tenants=False):
        return self.get(alarm_id, all_tenants)

    @pecan.expose('json')
    def get(self, alarm_id, all_tenants=False):
        all_tenants = bool_from_string(all_tenants)
        if all_tenants:
            enforce('get rca:all_tenants', pecan.request.headers,
                    pecan.request.enforcer, {})
        else:
            enforce('get rca', pecan.request.headers,
                    pecan.request.enforcer, {})

        LOG.info('received show rca with alarm id %s', alarm_id)
        return self.get_rca(alarm_id, all_tenants)

    @staticmethod
    def get_rca(alarm_id, all_tenants):
        try:
            graph_data = pecan.request.client.call(pecan.request.context,
                                                   'get_rca',
                                                   root=alarm_id,
                                                   all_tenants=all_tenants)
            LOG.info(graph_data)
            graph = json.loads(graph_data)
            return graph

        except Exception:
            LOG.exception('Failed to get RCA.')
            abort(404, 'Failed to get RCA')



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.datasources.nagios.parser import NagiosParser
from vitrage.datasources.nagios.properties import NagiosProperties
from vitrage.datasources.nagios.properties import NagiosTestStatus
from vitrage.tests.mocks import utils
from vitrage.tests.unit.datasources.nagios.nagios_base_test \
    import NagiosBaseTest


class NagiosParserTest(NagiosBaseTest):

    expected_service1 = {NagiosProperties.RESOURCE_NAME: 'compute-0-0.local',
                         NagiosProperties.SERVICE: 'CPU load',
                         NagiosProperties.STATUS: NagiosTestStatus.WARNING,
                         NagiosProperties.LAST_CHECK: '2016-02-09 13:05:32',
                         NagiosProperties.DURATION: ' 8d  2h 16m 33s',
                         NagiosProperties.ATTEMPT: '1/1',
                         NagiosProperties.STATUS_INFO:
                             'high CPU load\xa0'}

    expected_service2 = {NagiosProperties.RESOURCE_NAME: 'compute-0-1.local',
                         NagiosProperties.SERVICE: 'check_load',
                         NagiosProperties.STATUS: NagiosTestStatus.CRITICAL,
                         NagiosProperties.LAST_CHECK: '2016-02-16 14:27:06',
                         NagiosProperties.DURATION: ' 1d  0h 54m 59s',
                         NagiosProperties.ATTEMPT: '1/1',
                         NagiosProperties.STATUS_INFO:
                             'Critical Error\xa0'}

    expected_service3 = {NagiosProperties.RESOURCE_NAME: 'compute-0-0.local',
                         NagiosProperties.SERVICE: 'Disk IO SUMMARY',
                         NagiosProperties.STATUS: NagiosTestStatus.OK,
                         NagiosProperties.LAST_CHECK: '2016-02-17 15:21:22',
                         NagiosProperties.DURATION: '14d  1h 28m 34s',
                         NagiosProperties.ATTEMPT: '1/1',
                         NagiosProperties.STATUS_INFO:
                             'OK - 0.00 B/sec read, 1.84 MB/sec write, '
                             'IOs: 89.00/sec\xa0'}

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(NagiosParserTest, cls).setUpClass()

    def test_template_loader(self):
        # Setup
        fp = open(utils.get_resources_dir() + '/nagios/nagios-mock.html')
        nagios_html = fp.read()

        # Action
        nagios_services = NagiosParser().parse(nagios_html)

        # Test assertions
        self.assertTrue(nagios_services)
        self._assert_contains(self.expected_service1, nagios_services)
        self._assert_contains(self.expected_service2, nagios_services)
        self._assert_contains(self.expected_service3, nagios_services)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_db.sqlalchemy import session as db_session
from oslo_log import log
from sqlalchemy import and_, or_
from sqlalchemy.engine import url as sqlalchemy_url
from sqlalchemy import func

from vitrage.common.exception import VitrageInputError
from vitrage.entity_graph.mappings.operational_alarm_severity import \
    OperationalAlarmSeverity
from vitrage import storage
from vitrage.storage import base
from vitrage.storage.history_facade import HistoryFacadeConnection
from vitrage.storage.sqlalchemy import models
from vitrage.storage.sqlalchemy.models import Template

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class Connection(base.Connection):
    def __init__(self, url):
        options = dict(CONF.database.items())
        # set retries to 0 , since reconnection is already implemented
        # in storage.__init__.get_connection_from_config function
        options['max_retries'] = 0
        # add vitrage opts to database group
        for opt in storage.OPTS:
            options.pop(opt.name, None)
        self._engine_facade = db_session.EngineFacade(self._dress_url(url),
                                                      autocommit=True,
                                                      **options)
        self._active_actions = ActiveActionsConnection(self._engine_facade)
        self._events = EventsConnection(self._engine_facade)
        self._templates = TemplatesConnection(self._engine_facade)
        self._graph_snapshots = GraphSnapshotsConnection(self._engine_facade)
        self._webhooks = WebhooksConnection(
            self._engine_facade)
        self._alarms = AlarmsConnection(
            self._engine_facade)
        self._edges = EdgesConnection(
            self._engine_facade)
        self._changes = ChangesConnection(
            self._engine_facade)
        self._history_facade = HistoryFacadeConnection(
            self._engine_facade, self._alarms, self._edges, self._changes)

    @property
    def webhooks(self):
        return self._webhooks

    @property
    def active_actions(self):
        return self._active_actions

    @property
    def events(self):
        return self._events

    @property
    def templates(self):
        return self._templates

    @property
    def graph_snapshots(self):
        return self._graph_snapshots

    @property
    def alarms(self):
        return self._alarms

    @property
    def edges(self):
        return self._edges

    @property
    def changes(self):
        return self._changes

    @property
    def history_facade(self):
        return self._history_facade

    @staticmethod
    def _dress_url(url):
        # If no explicit driver has been set, we default to pymysql
        if url.startswith("mysql://"):
            url = sqlalchemy_url.make_url(url)
            url.drivername = "mysql+pymysql"
            return str(url)
        return url

    def disconnect(self):
        self._engine_facade.get_engine().dispose()

    def clear(self):
        engine = self._engine_facade.get_engine()
        for table in reversed(models.Base.metadata.sorted_tables):
            engine.execute(table.delete())
        engine.dispose()


class BaseTableConn(object):
    def __init__(self, engine_facade):
        super(BaseTableConn, self).__init__()
        self._engine_facade = engine_facade

    def bulk_create(self, items):
        if not items:
            return

        session = self._engine_facade.get_session()
        with session.begin():
            session.bulk_save_objects(items)

    def query_filter(self, model, **kwargs):
        session = self._engine_facade.get_session()
        query = session.query(model)
        for keyword, arg in kwargs.items():
            if arg is not None:
                query = query.filter(getattr(model, keyword) == arg)
        return query


class TemplatesConnection(base.TemplatesConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(TemplatesConnection, self).__init__(engine_facade)

    def create(self, template):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(template)

    def update(self, uuid, var, value):
        session = self._engine_facade.get_session()
        with session.begin():
            session.query(Template).filter_by(uuid=uuid).update({var: value})

    def query(self, name=None, file_content=None,
              uuid=None, status=None, status_details=None,
              template_type=None):
        query = self.query_filter(
            models.Template,
            name=name,
            file_content=file_content,
            uuid=uuid,
            status=status,
            status_details=status_details,
            template_type=template_type,
            )
        return query.all()

    def query_with_status_not(self, name, status):
        session = self._engine_facade.get_session()
        query = session.query(models.Template)
        query = query.filter(
            and_
            (
                models.Template.status != status,
                models.Template.name == name
            )
        )
        return query.first()

    def delete(self, name=None, uuid=None):
        query = self.query_filter(
            models.Template,
            name=name,
            uuid=uuid,
            )
        return query.delete()


class ActiveActionsConnection(base.ActiveActionsConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(ActiveActionsConnection, self).__init__(engine_facade)

    def create(self, active_action):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(active_action)

    def update(self, active_action):
        session = self._engine_facade.get_session()
        with session.begin():
            session.merge(active_action)

    def query(self,
              action_type=None,
              extra_info=None,
              source_vertex_id=None,
              target_vertex_id=None,
              action_id=None,
              score=None,
              trigger=None):
        query = self.query_filter(
            models.ActiveAction,
            action_type=action_type,
            extra_info=extra_info,
            source_vertex_id=source_vertex_id,
            target_vertex_id=target_vertex_id,
            action_id=action_id,
            score=score,
            trigger=trigger)
        return query.all()

    def query_similar(self, actions):
        """Query DB for all actions with same properties"""
        session = self._engine_facade.get_session()
        query = session.query(models.ActiveAction)

        filters = []
        for source, target, extra_info, action_type in actions:
            filters.append(
                and_(models.ActiveAction.action_type == action_type,
                     models.ActiveAction.extra_info == extra_info,
                     models.ActiveAction.source_vertex_id == source,
                     models.ActiveAction.target_vertex_id == target,))
        query = query.filter(or_(*filters))
        return query.all()

    def delete(self,
               action_type=None,
               extra_info=None,
               source_vertex_id=None,
               target_vertex_id=None,
               action_id=None,
               score=None,
               trigger=None):
        query = self.query_filter(
            models.ActiveAction,
            action_type=action_type,
            extra_info=extra_info,
            source_vertex_id=source_vertex_id,
            target_vertex_id=target_vertex_id,
            action_id=action_id,
            score=score,
            trigger=trigger)
        return query.delete()

    def bulk_delete(self, actions):
        if not actions:
            return
        session = self._engine_facade.get_session()
        query = session.query(models.ActiveAction)

        filters = []
        for trigger, action_id in actions:
            filters.append(
                and_(models.ActiveAction.trigger == trigger,
                     models.ActiveAction.action_id == action_id))
        query = query.filter(or_(*filters))
        return query.delete()


class WebhooksConnection(base.WebhooksConnection,
                         BaseTableConn):
    def __init__(self, engine_facade):
        super(WebhooksConnection, self).__init__(engine_facade)

    def create(self, webhook):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(webhook)

    def query(self,
              id=None,
              project_id=None,
              is_admin_webhook=None,
              url=None,
              headers=None,
              regex_filter=None):
        query = self.query_filter(
            models.Webhooks,
            id=id,
            project_id=project_id,
            is_admin_webhook=is_admin_webhook,
            url=url,
            headers=headers,
            regex_filter=regex_filter)
        return query.all()

    def delete(self, id=None):
        query = self.query_filter(models.Webhooks, id=id)
        return query.delete()


class EventsConnection(base.EventsConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(EventsConnection, self).__init__(engine_facade)

    def create(self, event):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(event)

    def update(self, event):
        session = self._engine_facade.get_session()
        with session.begin():
            session.merge(event)

    def get_last_event_id(self):
        session = self._engine_facade.get_session()
        query = session.query(models.Event.event_id)
        return query.order_by(models.Event.event_id.desc()).first()

    def get_replay_events(self, event_id):
        """Get all events that occurred after the specified event_id

        :rtype: list of vitrage.storage.sqlalchemy.models.Event
        """
        session = self._engine_facade.get_session()
        query = session.query(models.Event)
        query = query.filter(models.Event.event_id > event_id)
        return query.order_by(models.Event.event_id.asc()).all()

    def query(self,
              event_id=None,
              collector_timestamp=None,
              payload=None,
              gt_collector_timestamp=None,
              lt_collector_timestamp=None):
        """Yields a lists of events that match filters.

        :raises: vitrage.common.exception.VitrageInputError.
        :rtype: list of vitrage.storage.sqlalchemy.models.Event
        """

        if (event_id or collector_timestamp or payload) and \
           (gt_collector_timestamp or lt_collector_timestamp):
            msg = "Calling function with both specific event and range of " \
                  "events parameters at the same time "
            LOG.debug(msg)
            raise VitrageInputError(msg)

        query = self.query_filter(
            models.Event,
            event_id=event_id,
            collector_timestamp=collector_timestamp,
            payload=payload)

        query = self._update_query_gt_lt(gt_collector_timestamp,
                                         lt_collector_timestamp,
                                         query)

        return query.order_by(models.Event.collector_timestamp.desc()).all()

    @staticmethod
    def _update_query_gt_lt(gt_collector_timestamp,
                            lt_collector_timestamp,
                            query):
        if gt_collector_timestamp is not None:
            query = query.filter(models.Event.collector_timestamp >=
                                 gt_collector_timestamp)
        if lt_collector_timestamp is not None:
            query = query.filter(models.Event.collector_timestamp <=
                                 lt_collector_timestamp)
        return query

    def delete(self, event_id=None):
        """Delete all events older than event_id"""
        session = self._engine_facade.get_session()
        query = session.query(models.Event)
        if event_id:
            query = query.filter(models.Event.event_id < event_id)
        query.delete()


class GraphSnapshotsConnection(base.GraphSnapshotsConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(GraphSnapshotsConnection, self).__init__(engine_facade)

    def create(self, graph_snapshot):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(graph_snapshot)

    def update(self, graph_snapshot):
        session = self._engine_facade.get_session()
        with session.begin():
            session.merge(graph_snapshot)

    def query(self):
        query = self.query_filter(models.GraphSnapshot)
        return query.first()

    def query_snapshot_event_id(self):
        """Select the event_id of the stored snapshot"""
        session = self._engine_facade.get_session()
        query = session.query(models.GraphSnapshot.event_id)
        result = query.first()
        return result[0] if result else None

    def delete(self):
        """Delete all graph snapshots"""
        query = self.query_filter(models.GraphSnapshot)
        query.delete()


class AlarmsConnection(base.AlarmsConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(AlarmsConnection, self).__init__(engine_facade)

    def create(self, alarm):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(alarm)

    def update(self, vitrage_id, key, val):
        session = self._engine_facade.get_session()
        with session.begin():
            query = session.query(models.Alarm).filter(
                models.Alarm.vitrage_id == vitrage_id)
            query.update({getattr(models.Alarm, key): val})

    def end_all_alarms(self, end_time):
        session = self._engine_facade.get_session()
        query = session.query(models.Alarm).filter(
            models.Alarm.end_timestamp > end_time)
        query.update({models.Alarm.end_timestamp: end_time})

    def delete_expired(self, expire_by=None):
        session = self._engine_facade.get_session()
        query = session.query(models.Alarm)
        query = query.filter(models.Alarm.end_timestamp < expire_by)
        return query.delete()

    def delete(self,
               vitrage_id=None,
               start_timestamp=None,
               end_timestamp=None):
        query = self.query_filter(
            models.Alarm,
            vitrage_id=vitrage_id,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp)
        return query.delete()


class EdgesConnection(base.EdgesConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(EdgesConnection, self).__init__(engine_facade)

    def create(self, edge):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(edge)

    def update(self, source_id, target_id, end_timestamp):
        session = self._engine_facade.get_session()
        with session.begin():
            query = session.query(models.Edge).filter(and_(
                models.Edge.source_id == source_id,
                models.Edge.target_id == target_id))
            query.update({models.Edge.end_timestamp: end_timestamp})

    def end_all_edges(self, end_time):
        session = self._engine_facade.get_session()
        query = session.query(models.Edge).filter(
            models.Edge.end_timestamp > end_time)
        query.update({models.Edge.end_timestamp: end_time})

    def delete(self):
        query = self.query_filter(models.Edge)
        return query.delete()


class ChangesConnection(base.ChangesConnection, BaseTableConn):
    def __init__(self, engine_facade):
        super(ChangesConnection, self).__init__(engine_facade)

    def create(self, change):
        session = self._engine_facade.get_session()
        with session.begin():
            session.add(change)

    def add_end_changes(self, vitrage_ids, end_time):
        last_changes = self._get_alarms_last_change(vitrage_ids)
        for id, change in last_changes.items():
            change_row = \
                models.Change(
                    vitrage_id=id,
                    timestamp=end_time,
                    severity=OperationalAlarmSeverity.OK,
                    payload=change.payload)
            self.create(change_row)

    def _get_alarms_last_change(self, alarm_ids):
        session = self._engine_facade.get_session()
        query = session.query(func.max(models.Change.timestamp),
                              models.Change.vitrage_id,
                              models.Change.payload).\
            filter(models.Change.vitrage_id.in_(alarm_ids)).\
            group_by(models.Change.vitrage_id)

        rows = query.all()

        result = {}
        for change in rows:
            result[change.vitrage_id] = change

        return result

    def delete(self):
        query = self.query_filter(models.Change)
        return query.delete()



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory as ECategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.alarm_transformer_base import AlarmTransformerBase
from vitrage.datasources.prometheus import PROMETHEUS_DATASOURCE
from vitrage.datasources.prometheus.properties import get_alarm_update_time
from vitrage.datasources.prometheus.properties import get_label
from vitrage.datasources.prometheus.properties import PrometheusAlertLabels \
    as PAlertLabels
from vitrage.datasources.prometheus.properties \
    import PrometheusAlertProperties as PProps
from vitrage.datasources.prometheus.properties import PrometheusAlertStatus \
    as PAlertStatus
from vitrage.datasources.prometheus.properties \
    import PrometheusDatasourceProperties as PDProps
from vitrage.datasources import transformer_base as tbase
import vitrage.graph.utils as graph_utils

LOG = logging.getLogger(__name__)


class PrometheusTransformer(AlarmTransformerBase):

    def _create_snapshot_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_update_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_vertex(self, entity_event):
        metadata = {
            VProps.NAME: get_label(entity_event, PAlertLabels.ALERT_NAME),
            VProps.SEVERITY: get_label(entity_event, PAlertLabels.SEVERITY),
            PProps.STATUS: entity_event.get(PProps.STATUS),
        }
        return graph_utils.create_vertex(
            self._create_entity_key(entity_event),
            vitrage_category=ECategory.ALARM,
            vitrage_type=entity_event[DSProps.ENTITY_TYPE],
            vitrage_sample_timestamp=entity_event[DSProps.SAMPLE_DATE],
            entity_state=self._get_alarm_state(entity_event),
            update_timestamp=get_alarm_update_time(entity_event),
            metadata=metadata
        )

    def _create_snapshot_neighbors(self, entity_event):
        return self._create_prometheus_neighbors(entity_event)

    def _create_update_neighbors(self, entity_event):
        return self._create_prometheus_neighbors(entity_event)

    def _create_prometheus_neighbors(self, entity_event):
        graph_neighbors = entity_event.get(self.QUERY_RESULT, [])
        return [self._create_neighbor(entity_event,
                                      graph_neighbor[VProps.ID],
                                      graph_neighbor[VProps.VITRAGE_TYPE],
                                      EdgeLabel.ON,
                                      neighbor_category=ECategory.RESOURCE)
                for graph_neighbor in graph_neighbors]

    def _create_entity_key(self, entity_event):
        return tbase.build_key((ECategory.ALARM,
                                entity_event[DSProps.ENTITY_TYPE],
                                get_label(entity_event,
                                          PAlertLabels.ALERT_NAME),
                                str(entity_event.get(
                                    PDProps.ENTITY_UNIQUE_PROPS))))

    def get_vitrage_type(self):
        return PROMETHEUS_DATASOURCE

    def _ok_status(self, entity_event):
        return entity_event and \
            PAlertStatus.RESOLVED == entity_event.get(PProps.STATUS)

    @staticmethod
    def get_enrich_query(event):
        LOG.debug('event for enrich query: %s', event)
        entity_unique_props = event.get(PDProps.ENTITY_UNIQUE_PROPS)
        return entity_unique_props
